<?php

namespace Drupal\sdc_component_library\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\Core\Theme\ThemeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Yaml\Yaml;

/**
 * Provides route responses for component previews.
 */
class ComponentsController extends ControllerBase {

  /**
   * The theme extension list service.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  protected ThemeExtensionList $themeExtensionList;

  /**
   * The theme manager service.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected ThemeManagerInterface $themeManager;

  /**
   * The active theme name.
   *
   * @var string
   */
  protected string $activeTheme;

  /**
   * Constructs a ComponentsController object.
   *
   * @param \Drupal\Core\Extension\ThemeExtensionList $theme_extension_list
   *   The theme extension list service.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $theme_manager
   *   The theme manager service.
   */
  public function __construct(ThemeExtensionList $theme_extension_list, ThemeManagerInterface $theme_manager) {
    $this->themeExtensionList = $theme_extension_list;
    $this->themeManager = $theme_manager;
    $this->activeTheme = $this->themeManager->getActiveTheme()->getName();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('extension.list.theme'),
      $container->get('theme.manager')
    );
  }

  /**
   * Provides a page with a list of component previews.
   *
   * @return array
   *   A render array containing the component previews.
   */
  public function content() {
    // Get the active theme's path dynamically.
    $theme_path = $this->themeExtensionList->getPath($this->activeTheme);
    $components_path = $theme_path . '/components';

    // Use Symfony Finder to locate all *.story.twig files.
    $finder = new Finder();
    $finder->files()->in($components_path)->name('*.story.twig');

    // Array to hold the components content.
    $components = [];

    foreach ($finder as $file) {
      // Extract the relative path and file name.
      $relative_path = $file->getRelativePathname();

      // Expected path for the corresponding .component.yml file.
      $yaml_file = str_replace('.story.twig', '.component.yml', $file->getRealPath());

      // Determine if the component is an "atom", "molecule", etc.
      $type = explode('/', $relative_path)[0];

      // Default values in case the YAML file is missing or incomplete.
      $title = 'Unnamed Component';
      $description = '';
      $group = '';
      $props = [];
      $example_data = [];
      $libraryDependencies = [];

      // If the .component.yml file exists, parse it for component info.
      if (file_exists($yaml_file)) {
        $yaml_data = Yaml::parseFile($yaml_file);

        $title = $yaml_data['name'] ?? $title;
        $description = $yaml_data['description'] ?? $description;
        $group = $yaml_data['group'] ?? $group;

        if (isset($yaml_data['props']['properties'])) {
          $props = $yaml_data['props']['properties'];

          // Validate and store example data.
          foreach ($props as $key => $prop) {
            $example_data[$key] = $prop['examples'][0] ?? NULL;
          }
        }

        $libraryDependencies = $yaml_data['libraryDependencies'] ?? $libraryDependencies;
      }

      // Add component data to the array.
      $components[] = [
        'title' => $title,
        'description' => $description,
        'group' => $group,
        'type' => $type,
        'props' => $props,
        'example_data' => $example_data,
        'libraryDependencies' => $libraryDependencies,
        'theme' => 'component_preview_item',
        'twig_template' => '@' . $this->activeTheme . '/components/' . $relative_path,
      ];
    }

    // Return the render array with components and attach the CSS library.
    return [
      '#theme' => 'component_preview',
      '#components' => $components,
      '#attached' => [
        'library' => [
          'sdc_component_library/components_preview',
        ],
      ],
    ];
  }

}
